# Copyright (c) 2018, 2019, Oracle and/or its affiliates.  All rights reserved.
#
# This software is dual-licensed to you under the MIT License (MIT) and
# the Universal Permissive License (UPL).  See the LICENSE file in the root
# directory for license terms.  You may choose either license, or both.
#


class StateException(Exception):
    """This exception is raised if an object is in an inconsistent state."""

    def __init__(self, *args, **kwargs):
        super(StateException, self).__init__(args, kwargs)

class ArgumentException(ValueError):
    """This exception is raised if a method receives an invalid argument."""

    def __init__(self, *args, **kwargs):
        super(ArgumentException, self).__init__(args, kwargs)

class NetworkException(Exception):
    """This exception is raised if an IoT REST API request fails."""

    def __init__(self, *args, **kwargs):
        super(NetworkException, self).__init__(args, kwargs)

class SecurityException(Exception):
    """This exception is raised if credentials failure."""

    def __init__(self, *args, **kwargs):
        super(SecurityException, self).__init__(args, kwargs)

class TransportException(Exception):
    """This exception is raised if an HTTP error occurs."""

    __slots__ = (
        'statuscode',
        'message',
        'method',
        'path',
        'headers',
        'data'
    )

    def __init__(self, status=None, message=None, method=None, path=None, httpresponse=None,
                 *args, **kwargs):
        """

        :param statuscode: The HTTP error status code
        :param message:
        :param method:
        :param path:
        :param httpresponse:
        :param args:
        :param kwargs:
        """

        super(TransportException, self).__init__(args, kwargs)
        self.status = status
        self.message = message
        self.method = method
        self.path = path
        self.data = httpresponse.getData() if httpresponse is not None else None
        self.headers = httpresponse.getHeaders() if httpresponse is not None else None

    def getStatusCode(self):
        """
        Return the HTTP error status code causing this exception.

        :return: The HTTP error status code causing this exception
        """
        return self.status

    def getResponseData(self):
        return self.data

    def getResponseHeader(self, header):
        if self.headers is not None and header in self.headers:
            return self.headers[header]
        return None

    def getMessage(self):
        return self.message

class SocketException(Exception):
    """This exception is raised if a socket error occurs."""

class SocketTimeoutException(Exception):
    """This exception is raised if a socket timeout error occurs."""

class HttpException(Exception):
    """This exception is raised if an HTTPError or HTTTPWarning occurs."""
